<?php

/**
 * Třída implementující práci se soubory.
 */
class FileLoader implements IFileLoader {

    public $PATH_TO_CURRENT_USERS = './internal/users.txt';
    //define('PATH_TO_INCOMING_USERS,','./external/users.txt');
    public $PATH_TO_INCOMING_USERS = './external/users.txt';

    public $PATH_TO_INCOMING_ROOMS = './external/rooms.txt';
    public $PATH_TO_CURRENT_ROOMS = './internal/rooms.txt';

    private $PATH_TO_INCOMING_ROOM = './external/';
    private $PATH_TO_CURRENT_ROOM = './internal/';

    // public functions ------------------------------------------------------

    /**
     * Načte uživatele z textového souboru. Očekávaný formát řádku je:
     * eppn;jméno;příjmení
     * Pokud se soubor nepodaří načíst, je vyvolána výjimka.
     * @param string $pathAndFile - cesta k souboru odkud se data načítají
     * @return pole instancí User
     */
    public function loadUsers($pathAndFile) {

        $methodName = 'loadUsers';
        $this->logDebug('Entering method '.$methodName.' with param: ['.$pathAndFile.']', $methodName);

        //check file
        $this->checkFile($pathAndFile);

        $result = array();
        $lines = file($pathAndFile);
        foreach ($lines as $line_num => $line) {
            //check line format
            $this->isUserLineWellFormated($line);
            //split the line by delimeter a semicolon in this case
            $lineArray = explode(';',$line);

            $user = new User();
            //suppose that the line-format is as following: eppn;name;lastname
            $user->setEppn(trim($lineArray[0]));
            $user->setName(trim($lineArray[1]));
            $user->setLastname(trim($lineArray[2]));
            $user->setEmail(trim($lineArray[3]));
            $result[] = $user;
            $this->logDebug('Loaded user: ['.$user->toString().']', $methodName);
        }

        return $result; //vrati pole User

    }

    /**
     * Načte místnosti z textového souboru. Jeden řádek reprezentuje jednu
     * místnost (její název).
     * Pokud se soubor nepodaří načíst, je vyvolána výjimka.
     * @param string $pathAndFile
     * @return pole instancí Room
     */
    public function loadRooms($pathAndFile) {

        $methodName = 'loadRooms';
        $this->logDebug('Entering method '.$methodName.' with param: ['.$pathAndFile.']', $methodName);

        $this->checkFile($pathAndFile);

        $result = array();
        $lines = file($pathAndFile);
        foreach ($lines as $line_num => $line) {
            $room = new Room($line);
            $result[] = $room;
            $this->logDebug('Loaded room: ['.$room->toString().']', $methodName);
        }

        return $result; // returns array of Room
    }

    /**
     * Načte uživatele z textového souboru. Očekávný formát řádku je:
     * eppn
     * Pokud se soubor nepodaří načíst je vyvolána výjimka.
     * @param string $pathAndFile
     * @return pole instancí User
     */
    public function loadUsersOfRoom($pathAndFile) {

        $methodName = 'loadUsersOfRoom';
        $this->logDebug('Entering method '.$methodName.' with param: ['.$pathAndFile.']', $methodName);

        $this->checkFile($pathAndFile);

        $result = array();
        $lines = file($pathAndFile);
        foreach ($lines as $line_num => $line) {
            $user = new User();
            //suppose that the line-format is as following: eppn
            $user->setEppn(trim($line));
            $result[] = $user;
            $this->logDebug('Loaded user ['.$user->toString().'] from file ['.$pathAndFile.']', $methodName);
        }
        return $result; //vrati pole User
    }

    // private functions -----------------------------------------------------

    /**
     * Kontroluje zda-li je soubor přístupný.
     * @param string $pathAndFile
     * @return true nebo false
     */
    private function isFileAccessible($pathAndFile) {
        return (file_exists($pathAndFile) && !is_dir($pathAndFile));
    }


    //ToDo refactor name !
    /**
     * Kontroluje správný formát řádku obsahující informace o uživateli.
     * Očekáváný formát je:
     * eppn;jméno;příjmení
     * @param string $line kontrolovaný řádek
     */
    private function isUserLineWellFormated($line) {

        $methodName = 'isUserLineWellFormated';
        $this->logDebug('Entering method '.$methodName, $methodName);

        $lineArray = explode(';',$line);
        //we expect something like: eppn;name;surname
        if (count($lineArray) != 4) {
            $this->logError('Wrong format of line: ['.$line.']', $methodName);
        }
    }

    /**
     * Kontroluje přístupnost souboru.
     * @param string $pathAndFile
     */
    private function checkFile($pathAndFile) {

        $methodName = 'checkFile';
        $this->logDebug('Entering method '.$methodName.' with param: ['.$pathAndFile.']', $methodName);

        if (!$this->isFileAccessible($pathAndFile)) {
            $this->logError('Inaccesible file! Accessing file ['.$pathAndFile.'] failed.', $methodName);
        }
        $this->logDebug('Leaving method '.$methodName, $methodName);
    }


    public function recurse_copy($src,$dst) {
        $methodName = 'recurse_copy';
        $this->logDebug('Entering method '.$methodName.' with param: ['.$src.', '.$dst.']', $methodName);
        $dir = opendir($src);
        @mkdir($dst);
        while(false !== ( $file = readdir($dir)) ) {
            if (( $file != '.' ) && ( $file != '..' )) {
                if ( is_dir($src . '/' . $file) ) {
                    recurse_copy($src . '/' . $file,$dst . '/' . $file);
                }
                else {
                    copy($src . '/' . $file,$dst . '/' . $file);
                }
            }
        }
        closedir($dir);
        $this->logDebug('Leaving method '.$methodName, $methodName);
    }

    public function deleteAllFiles($pathToFolder) {
        $methodName = 'deleteAllFiles';
        $this->logDebug('Entering method '.$methodName.' with param: ['.$pathToFolder.']', $methodName);
        $files = glob("$pathToFolder*.*");
        foreach ($files as $file) {
            unlink($file);

        }
        $this->logDebug('Leaving method '.$methodName, $methodName);
    }

    public function logDebug($description, $methodName) {
        SysLogger::logDebug($description, 'FileLoader', $methodName);
    }

    public function logError($description, $methodName) {
        SysLogger::logError($description, 'FileLoader', $methodName);
    }
}
?>
